/** @module avec3 */
export function set3(a: any, i: any, x: any, y: any, z: any): void;
/**
 * Sets a vector to another vector.
 * @param {avec3} a
 * @param {number} i
 * @param {avec3} b
 * @param {number} j
 */
export function set(a: avec3, i: number, b: avec3, j: number): void;
/**
 * Compares two vectors.
 * @param {avec3} a
 * @param {number} i
 * @param {avec3} b
 * @param {number} j
 * @returns {boolean}
 */
export function equals(a: avec3, i: number, b: avec3, j: number): boolean;
/**
 * Adds a vector to another.
 * @param {avec3} a
 * @param {number} i
 * @param {avec3} b
 * @param {number} j
 */
export function add(a: avec3, i: number, b: avec3, j: number): void;
/**
 * Subtracts a vector from another.
 * @param {avec3} a
 * @param {number} i
 * @param {avec3} b
 * @param {number} j
 */
export function sub(a: avec3, i: number, b: avec3, j: number): void;
/**
 * Scales a vector by a number.
 * @param {avec3} a
 * @param {number} i
 * @param {number} s
 */
export function scale(a: avec3, i: number, s: number): void;
/**
 * Adds two vectors after scaling the second one.
 * @param {avec3} a
 * @param {number} i
 * @param {avec3} b
 * @param {number} j
 * @param {number} s
 */
export function addScaled(a: avec3, i: number, b: avec3, j: number, s: number): void;
/**
 * Calculates the dot product of two vectors.
 * @param {avec3} a
 * @param {number} i
 * @param {avec3} b
 * @param {number} j
 */
export function dot(a: avec3, i: number, b: avec3, j: number): number;
/**
 * Calculates the cross product of two vectors.
 * @param {avec3} a
 * @param {number} i
 * @param {avec3} b
 * @param {number} j
 */
export function cross(a: avec3, i: number, b: avec3, j: number): void;
/**
 * Calculates the length of a vector.
 * @param {avec3} a
 * @param {number} i
 * @returns {number}
 */
export function length(a: avec3, i: number): number;
/**
 * Calculates the squared length of a vector.
 * @param {avec3} a
 * @param {number} i
 * @returns {number}
 */
export function lengthSq(a: avec3, i: number): number;
/**
 * Normalises a vector.
 * @param {avec3} a
 * @param {number} i
 */
export function normalize(a: avec3, i: number): void;
/**
 * Calculates the distance between two vectors.
 * @param {avec3} a
 * @param {number} i
 * @param {avec3} b
 * @param {number} j
 * @returns {number}
 */
export function distance(a: avec3, i: number, b: avec3, j: number): number;
/**
 * Calculates the squared distance between two vectors.
 * @param {avec3} a
 * @param {number} i
 * @param {avec3} b
 * @param {number} j
 * @returns {number}
 */
export function distanceSq(a: avec3, i: number, b: avec3, j: number): number;
/**
 * Limits a vector to a length.
 * @param {avec3} a
 * @param {number} i
 * @param {number} len
 */
export function limit(a: avec3, i: number, len: number): void;
/**
 * Linearly interpolates between two vectors.
 * @param {avec3} a
 * @param {number} i
 * @param {avec3} b
 * @param {number} j
 * @param {number} t
 */
export function lerp(a: avec3, i: number, b: avec3, j: number, t: number): void;
/**
 * Prints a vector to a string.
 * @param {avec3} a
 * @param {number} i
 * @param {number} [precision=4]
 * @returns {string}
 */
export function toString(a: avec3, i: number, precision?: number): string;
