/**
 * Returns a new quat at 0, 0, 0, 1.
 * @returns {quat}
 */
export function create(): quat;
/**
 * Sets a quaternion to the identity quaternion.
 * @param {quat} a
 * @returns {quat}
 */
export function identity(a: quat): quat;
/**
 * Returns a copy of a quaternion.
 * @param {quat} a
 * @returns {quat}
 */
export function copy(a: quat): quat;
/**
 * Multiplies one quaternion by another.
 * @param {quat} a
 * @param {quat} b
 * @returns {quat}
 */
export function mult(a: quat, b: quat): quat;
/**
 * Inverts a quaternion.
 * @param {quat} a
 * @returns {quat}
 */
export function invert(a: quat): quat;
/**
 * Conjugates a quaternion.
 * @param {quat} a
 * @returns {quat}
 */
export function conjugate(a: quat): quat;
/**
 * Calculates the length of a quaternion.
 * @param {quat} a
 * @returns {quat}
 */
export function length(a: quat): quat;
/**
 * Normalizes a quaternion.
 * @param {quat} a
 * @returns {quat}
 */
export function normalize(a: quat): quat;
/**
 * Calculates the dot product of two quaternions.
 * @param {quat} a
 * @param {quat} b
 * @returns {quat}
 */
export function dot(a: quat, b: quat): quat;
/**
 * Set euler angles to a quaternion. Assumes XYZ rotation order.
 * @param {quat} a
 * @param {euler} e
 * @returns {quat}
 */
export function fromEuler(a: quat, e: euler): quat;
/**
 * Set the angle at an axis of a quaternion.
 * @param {quat} a
 * @param {vec3} v
 * @param {Radians} r
 * @returns {quat}
 */
export function fromAxisAngle(a: quat, v: typeof vec3, r: Radians): quat;
/**
 * @private
 */
export function _fromMat39(a: any, m0: any, m1: any, m2: any, m3: any, m4: any, m5: any, m6: any, m7: any, m8: any): any;
/**
 * Sets a quaternion from orthonormal base xyz.
 * @param {quat} a
 * @param {vec3} x
 * @param {vec3} y
 * @param {vec3} z
 * @returns {quat}
 */
export function fromAxes(a: quat, x: typeof vec3, y: typeof vec3, z: typeof vec3): quat;
/**
 * Sets a quaternion to a 3x3 matrix.
 * @param {quat} a
 * @param {mat3} m
 * @returns {quat}
 */
export function fromMat3(a: quat, m: mat3): quat;
/**
 * Sets a quaternion to a 4x4 matrix.
 * @param {quat} a
 * @param {mat4} m
 * @returns {quat}
 */
export function fromMat4(a: quat, m: mat4): quat;
/**
 * Spherical linear interpolates between two quaternions.
 * @param {quat} a
 * @param {quat} b
 * @param {number} t
 * @returns {quat}
 */
export function slerp(a: quat, b: quat, t: number): quat;
/**
 * Sets a quaternion to another quaternion.
 * @param {quat} a
 * @param {quat} b
 * @returns {quat}
 */
export const set: typeof vec4.set;
/**
 * Compares two quaternions.
 * @param {quat} a
 * @param {quat} b
 * @returns {boolean}
 */
export const equals: typeof vec4.equals;
export function fromTo(a: any, v: any, w: any): any;
/**
 * Prints a quaternion to a string.
 * @param {quat} a
 * @param {number} precision
 * @returns {quat}
 */
export const toString: typeof vec4.toString;
import * as vec3 from "./vec3.js";
import * as vec4 from "./vec4.js";
