/**
 * Linear interpolation between two numbers.
 * @param {number} a
 * @param {number} b
 * @param {number} t
 * @returns {number}
 */
export function lerp(a: number, b: number, t: number): number;
/**
 * Clamps a number between two numbers.
 * @param {number} n
 * @param {number} min
 * @param {number} max
 * @returns {number}
 */
export function clamp(n: number, min: number, max: number): number;
/**
 * Smooth Hermite interpolation between 0 and 1
 * @param {number} n
 * @param {number} min
 * @param {number} max
 * @returns {number}
 */
export function smoothstep(n: number, min: number, max: number): number;
/**
 * Maps a number from one range to another.
 * @param {number} n
 * @param {number} inStart
 * @param {number} inEnd
 * @param {number} outStart
 * @param {number} outEnd
 * @returns {number}
 */
export function map(n: number, inStart: number, inEnd: number, outStart: number, outEnd: number): number;
/**
 * Transforms degrees into radians.
 * @param {Degrees} degrees
 * @returns {Radians}
 */
export function toRadians(degrees: Degrees): Radians;
/**
 * Transforms radians into degrees.
 * @param {Radians} radians
 * @returns {Degrees}
 */
export function toDegrees(radians: Radians): Degrees;
/**
 * Returns the sign of a number.
 * @param {number} n
 * @returns {number}
 */
export function sign(n: number): number;
/**
 * Check if a number is a power of two
 * @param {number} a
 * @returns {boolean}
 */
export function isPowerOfTwo(a: number): boolean;
/**
 * Returns the next highest power of two.
 * @param {number} n
 * @returns {number}
 */
export function nextPowerOfTwo(n: number): number;
/** @module utils */
/**
 * @typedef {number} Degrees
 */
/**
 * @typedef {number} Radians
 */
/**
 * @constant {number}
 */
export const EPSILON: 0.000001;
export type Degrees = number;
export type Radians = number;
