/** @module vec2 */
/**
 * Returns a new vec2 at 0, 0, 0.
 * @returns {vec2}
 */
export function create(): vec2;
/**
 * Returns a copy of a vector.
 * @param {vec2} a
 * @returns {vec2}
 */
export function copy(a: vec2): vec2;
/**
 * Sets a vector to another vector.
 * @param {vec2} a
 * @param {vec2} b
 * @returns {vec2}
 */
export function set(a: vec2, b: vec2): vec2;
/**
 * Compares two vectors.
 * @param {vec2} a
 * @param {vec2} b
 * @returns {boolean}
 */
export function equals(a: vec2, b: vec2): boolean;
/**
 * Add a vector to another.
 * @param {vec2} a
 * @param {vec2} b
 * @returns {vec2}
 */
export function add(a: vec2, b: vec2): vec2;
/**
 * Subtracts a vector from another.
 * @param {vec2} a
 * @param {vec2} b
 * @returns {vec2}
 */
export function sub(a: vec2, b: vec2): vec2;
/**
 * Scales a vector by a number.
 * @param {vec2} a
 * @param {number} s
 * @returns {vec2}
 */
export function scale(a: vec2, s: number): vec2;
/**
 * Adds two vectors after scaling the second one.
 * @param {vec2} a
 * @param {vec2} b
 * @param {number} s
 * @returns {vec2}
 */
export function addScaled(a: vec2, b: vec2, s: number): vec2;
/**
 * Calculates the dot product of two vectors.
 * @param {vec2} a
 * @param {vec2} b
 * @returns {number}
 */
export function dot(a: vec2, b: vec2): number;
/**
 * Calculates the length of a vector.
 * @param {vec2} a
 * @returns {number}
 */
export function length(a: vec2): number;
/**
 * Calculates the squared length of a vector.
 * @param {vec2} a
 * @returns {number}
 */
export function lengthSq(a: vec2): number;
/**
 * Normalises a vector.
 * @param {vec2} a
 * @returns {vec2}
 */
export function normalize(a: vec2): vec2;
/**
 * Calculates the distance between two vectors.
 * @param {vec2} a
 * @param {vec2} b
 * @returns {number}
 */
export function distance(a: vec2, b: vec2): number;
/**
 * Calculates the squared distance between two vectors.
 * @param {vec2} a
 * @param {vec2} b
 * @returns {number}
 */
export function distanceSq(a: vec2, b: vec2): number;
/**
 * Limits a vector to a length.
 * @param {vec2} a
 * @param {number} len
 * @returns {vec2}
 */
export function limit(a: vec2, len: number): vec2;
/**
 * Linearly interpolates between two vectors.
 * @param {vec2} a
 * @param {vec2} b
 * @param {number} t
 * @returns {vec2}
 */
export function lerp(a: vec2, b: vec2, t: number): vec2;
/**
 * Prints a vector to a string.
 * @param {vec2} a
 * @param {number} [precision=4]
 * @returns {string}
 */
export function toString(a: vec2, precision?: number): string;
