/** @module vec4 */
/**
 * Returns a new vec4 at 0, 0, 0, 1.
 * @returns {vec4}
 */
export function create(): vec4;
/**
 * Returns a copy of a vector.
 * @param {vec4} a
 * @returns {vec4}
 */
export function copy(a: vec4): vec4;
/**
 * Sets a vector to another vector.
 * @param {vec4} a
 * @param {vec4} b
 * @returns {vec4}
 */
export function set(a: vec4, b: vec4): vec4;
/**
 * Compares two vectors.
 * @param {vec4} a
 * @param {vec4} b
 * @returns {boolean}
 */
export function equals(a: vec4, b: vec4): boolean;
/**
 * Adds a vector to another.
 * @param {vec4} a
 * @param {vec4} b
 * @returns {vec4}
 */
export function add(a: vec4, b: vec4): vec4;
/**
 * Subtracts a vector from another.
 * @param {vec4} a
 * @param {vec4} b
 * @returns {vec4}
 */
export function sub(a: vec4, b: vec4): vec4;
/**
 * Scales a vector by a number.
 * @param {vec4} a
 * @param {number} s
 * @returns {vec4}
 */
export function scale(a: vec4, s: number): vec4;
/**
 * Adds two vectors after scaling the second one.
 * @param {vec4} a
 * @param {vec4} b
 * @param {number} s
 * @returns {vec4}
 */
export function addScaled(a: vec4, b: vec4, s: number): vec4;
/**
 * Create a vec4 from vec3.
 * @param {vec4} a
 * @param {vec3} b
 * @returns {vec4}
 */
export function fromVec3(a: vec4, b: vec3): vec4;
/**
 * Multiplies a vector with a matrix.
 * @param {vec4} a
 * @param {mat4} m
 * @returns {vec4}
 */
export function multMat4(a: vec4, m: mat4): vec4;
/**
 * Linearly interpolates between two vectors.
 * @param {vec4} a
 * @param {vec4} b
 * @param {number} t
 * @returns {vec4}
 */
export function lerp(a: vec4, b: vec4, t: number): vec4;
/**
 * Prints a vector to a string.
 * @param {vec4} a
 * @param {number} [precision=4]
 * @returns {string}
 */
export function toString(a: vec4, precision?: number): string;
